/****************************************************************************
    MP4 input module

    Copyright (C) 2017 Krzysztof Nikiel

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/


/* 2022.03.19 AP: modified to work with POCC and multithreading */

#include <stdint.h>

#pragma pack(push,2)

typedef struct
{
    uint32_t ctime, mtime;
    uint32_t samplerate;
    uint16_t sampleratefrac;
    uint16_t format;
    // total sound samples
    uint32_t samples;
    uint16_t channels;
    // sample depth
    uint16_t bits;
    uint8_t esflags[2];
    // buffer config
    uint16_t buffersize;
    uint32_t bitratemax;
    uint32_t bitrateavg;
    struct
    {
        uint32_t *data;
        uint32_t ents;
        int current;
        int maxsize;
    } frame;
    // AudioSpecificConfig data:
    struct
    {
        uint8_t buf[24]; /* there are USAC files with 23 (identification only) */
        int size;
    } asc;
    uint32_t dataofs;
    uint32_t datasize;
    struct {
        int size;
        uint8_t *data;
    } bitbuf;
    struct {
        uint8_t header;
        uint8_t tags;
        uint8_t errors;
    } verbose;
} mp4config_t;

#pragma pack(pop)

//extern mp4config_t mp4config;

FILE* mp4read_open(char *name, mp4config_t *config, int *err);
int _mp4read_open(FILE *stream, mp4config_t *config);
int mp4read_seek(FILE *stream, mp4config_t *config, int framenum);
int mp4read_frame(FILE *stream, mp4config_t *config);
int mp4read_close(FILE *stream, mp4config_t *config);

int _mp4read_esds_atom(FILE *f, int size, mp4config_t *config);
